
/* Chrysalide - Outil d'analyse de fichiers binaires
 * instruction-int.h - prototypes pour la définition générique interne des instructions
 *
 * Copyright (C) 2008-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ARCH_INSTRUCTION_INT_H
#define _ARCH_INSTRUCTION_INT_H


#include "archbase.h"
#include "instruction.h"
#include "translate.h"
#include "../common/dllist.h"



/* Liste les registres lus et écrits par l'instruction. */
typedef void (* get_instruction_rw_regs_fc) (const GArchInstruction *, GArchRegister ***, size_t *, GArchRegister ***, size_t *);

/* Indique l'encodage d'une instruction de façon détaillée. */
typedef const char * (* get_instruction_encoding_fc) (const GArchInstruction *);

/* Ajoute à un tampon GLib le contenu de l'instance spécifiée. */
typedef GBufferLine * (* print_instruction_fc) (const GArchInstruction *, GCodeBuffer *, MemoryDataSize, const GBinContent *, AsmSyntax);

/* Fournit le nom humain de l'instruction manipulée. */
typedef const char * (* get_instruction_keyword_fc) (GArchInstruction *, AsmSyntax );

/* Informe sur une éventuelle référence à une autre instruction. */
typedef InstructionLinkType (* get_instruction_link_fc) (const GArchInstruction *, vmpa_t *);


/* Définition générique d'une instruction d'architecture (instance) */
struct _GArchInstruction
{
    GObject parent;                         /* A laisser en premier        */

    DL_LIST_ITEM(flow);                     /* Maillon de liste chaînée    */

    phys_t max_displayed_len;               /* Quantité de code affichée   */

    const instr_hook_fc *hooks;             /* Traitements complémentaires */

    mrange_t range;                         /* Emplacement en mémoire      */

    GArchOperand **operands;                /* Liste des opérandes         */
    size_t operands_count;                  /* Nbre. d'opérandes utilisées */

    instr_link_t *from;                     /* Origines des références     */
    size_t from_count;                      /* Nombre de ces origines      */
    instr_link_t *to;                       /* Instructions visées         */
    size_t to_count;                        /* Nombre de ces destinations  */
    GRWLock link_access;                    /* Verrou de protection        */
#ifndef NDEBUG
    gint hold_link_access;                  /* Suivi des verrouillages     */
#endif

    ArchInstrFlag flags;                    /* Informations complémentaires*/

    //get_instruction_rw_regs_fc get_rw_regs; /* Liste des registres liés    */
    //print_instruction_fc print;             /* Imprime l'ensemble          */
    //get_instruction_keyword_fc get_key;     /* Texte humain équivalent     */
    //is_instruction_return_fc is_return;     /* Retour de fonction ou pas ? */
    //decomp_instr_fc decomp;                 /* Procédure de décompilation  */

};


/* Définition générique d'une instruction d'architecture (classe) */
struct _GArchInstructionClass
{
    GObjectClass parent;                    /* A laisser en premier        */

    get_instruction_encoding_fc get_encoding; /* Obtention de l'encodage   */
    get_instruction_keyword_fc get_keyword; /* Texte humain équivalent     */

    print_instruction_fc print;             /* Imprime l'ensemble          */

};


#define ainstr_list_last(head) dl_list_last(head, GArchInstruction, flow)
#define ainstr_list_prev_iter(iter, head) dl_list_prev_iter(iter, head, GArchInstruction, flow)
#define ainstr_list_next_iter(iter, head) dl_list_next_iter(iter, head, GArchInstruction, flow)
#define ainstr_list_add_tail(new, head) dl_list_add_tail(new, head, GArchInstruction, flow)
#define ainstr_list_merge(head1, head2) dl_list_merge(head1, head2, GArchInstruction, flow)
#define ainstr_list_for_each(pos, head) dl_list_for_each(pos, head, GArchInstruction, flow)



#endif  /* _ARCH_INSTRUCTION_INT_H */
